/* dmpiatom - Atomic types */

#ifndef _DMPIATOM_INCLUDE
#define _DMPIATOM_INCLUDE

/*****************************************************************************
*                           MPI ATOMIC DATA STRUCTURES                             *
*****************************************************************************/

/* insure that this is at least 32 bits */
typedef unsigned long uint32;

typedef struct {
    int np;			/* Number of processes in group */
    uint32 *lrank_to_grank;	/* Mapping from local to "global" ranks */
} MPIR_GROUP;

typedef unsigned long MPIR_CONTEXT;
#define  MPIR_PT2PT_CONTEXT 0
#define  MPIR_COLL_CONTEXT  1

typedef enum { 
    MPI_ERRORS_FATAL = 0, MPI_ERRORS_RETURN, MPI_ERRORS_CALL_USER } 
MPI_ERROR_MODE;

typedef enum { MPIR_INTRA=0, MPIR_INTER } MPIR_COMM_TYPE;
typedef struct _MPIR_COMMUNICATOR {
    MPIR_COMM_TYPE comm_type;	/* inter or intra */
    MPIR_GROUP   *group;	/* group associated with communicator */

    MPIR_CONTEXT context_pt2pt,	/* context for point-to-point */
                 context_coll;  /* contect for collective operations */
    void         *comm_cache;	/* Hook for communicator cache */
    void         *attribute_cache;   /* Hook for attribute cache */
    MPI_ERROR_MODE  error_mode;      /* Error mode */
    int          (*error_handler)();  /* User routine to call */
} MPIR_COMMUNICATOR;

typedef enum {
    NO = 0, YES
} MPIR_BOOL;

typedef enum {
    MPIR_INT, MPIR_FLOAT, MPIR_DOUBLE, MPIR_LONG, MPIR_SHORT, MPIR_CHAR,
    MPIR_BYTE, MPIR_CONTIG, MPIR_VECTOR, MPIR_HVECTOR, MPIR_INDEXED,
    MPIR_HINDEXED, MPIR_STRUCT
} MPIR_NODETYPE;

#include "mpi_bc.h"
#endif
