/*  dmpi.c - routines in mpir that are called by the device */

#include "mpi.h"
/* #include "dmpi.h" */

#define MPIR_MIN(a,b) (a) < (b) ? (a) : (b)

/* called by device when a message arrives.  Returns 1 if there is posted
 * receive, 0 otherwise.
 */

DMPI_msg_arrived( src, tag, context_id, dmpi_recv_handle, status )
int               src, tag, *status;
MPIR_CONTEXT      context_id;
MPIR_RHANDLE      **dmpi_recv_handle;
{
    int          found;
    MPIR_RHANDLE *handleptr;

    MPIR_search_queue(posted_recvs, src, tag, context_id, &found, 1, 
		      dmpi_recv_handle);
    if ( found )
    {
	printf("DMPI_msg_arrived:  posted receive found\n");
	*status = 1;	
	/*
	MPID_complete_recv( &((*dmpi_recv_handle)->dev_rhandle),
			    dmpi_recv_handle);
	 */
    }
    else
    {
	printf("DMPI_msg_arrived:  no posted receive found\n");
	/* allocate handle and put in unexpected queue */
	*dmpi_recv_handle       = ( MPIR_RHANDLE * ) SBalloc ( MPIR_rhandles );
	handleptr         	= *dmpi_recv_handle;
	handleptr->source 	= src;
	    /* THIS IS WRONG!!! */
	handleptr->comm   	= MPI_COMM_WORLD; /* comm */;
	handleptr->persistence  = MPI_EPHEMERAL;	/* ??? */
	handleptr->datatype     = MPI_BYTE;	/* ??? */
	handleptr->completed    = NO;
	MPID_alloc_recv_handle( &(handleptr->dev_rhandle) );
	
	MPIR_enqueue( unexpected_recvs, (void * ) *dmpi_recv_handle,
                      MPIR_QSHANDLE );
	*status = 0;
    }
}

/* search specified queue for message matching criteria */
/* CHANGE TO ALLOW SEARCH AND DELETE.  NEED TO INSURE THAT ONCE MATCHED,
   A POSTED MESSAGE DOESN'T MATCH AGAIN.  Note that we need to match without
   deleting so that we can implement probe 

   A flag of 1 causes a successful search to delete the element found 
 */
MPIR_search_queue( queue, src, tag, context_id, found, flag, handleptr )
MPIR_QHDR 	  *queue;
int       	  src, tag;
MPIR_CONTEXT      context_id;
int               *found;
MPIR_RHANDLE      **handleptr;
int               flag;
{
    MPIR_QEL     *p;
    MPIR_RHANDLE *q;

    p      = queue->first;
    *found = 0;
    while ((!*found) && p)
    {
	q = ( MPIR_RHANDLE * ) p->ptr;
	/* The communicator test is not yet correct */
	if ( /* ( context_id == q->comm->context_pt2pt ) && */
	     ( tag  == q->tag || q->tag == MPI_ANY_TAG ) &&
	     ( src  == q->source || q->source == MPI_ANY_SOURCE ))
	{
	    *found = 1;
	    *handleptr = q;
	    if (flag) {
		if ( p->next != NULL )
		    p->next->prev = p->prev;
		if ( p->prev != NULL )
		    p->prev->next = p->next;
		queue->currlen--;
		printf("fix up header prts!\n");
		}
	}
	p   = p->next;
    }
}

/* get length of data to be sent
 */
DMPI_get_totallen( handle, len )
MPIR_SHANDLE *handle;
int          *len;
{
    *len = handle->datatype->extent;
}

/* device has send buffer ready, wants mpi to copy data to it
 */
DMPI_get_into_contig( handle, addr, maxlen, actlen )
MPIR_SHANDLE *handle;
void         *addr;
int          maxlen;
int          *actlen;
{
    *actlen = MPIR_MIN(handle->datatype->extent,maxlen);
    memcpy( addr, handle->bufadd, *actlen );
}

/* device will copy mpi's contiguous buffer, wants to know where it is
 */
DMPI_get_from_contig( handle, addr, maxlen, actlen )
MPIR_SHANDLE *handle;
void         **addr;
int          maxlen;
int          *actlen;
{
    if ( maxlen < handle->datatype->extent )
	MPIR_error( "DMPI_get_from_contig:  Device's buffer too small" );
    else
    {
	*addr   = handle->bufadd;
	*actlen = handle->datatype->extent;
    }
}

/* device will put data into mpi's contiguous buffer, wants to know where it is
 */
DMPI_put_into_contig( handle, addr, maxlen, actlen )
MPIR_RHANDLE *handle;
void         **addr;
int          maxlen;
int          *actlen;
{
    if ( maxlen > handle->datatype->extent )
	MPIR_error( "DMPI_put_from_contig:  MPI's buffer too small" );
    else
    {
	*addr   = handle->bufadd;
	*actlen = handle->datatype->extent;
    }
}

/* device has contiguous buffer ready, wants mpi to copy data from it
 */
DMPI_put_from_contig( handle, addr, maxlen )
MPIR_RHANDLE *handle;
void         *addr;
int          maxlen;
{
    if ( maxlen > handle->datatype->extent )
	MPIR_error( "DMPI_put_from_contig:  MPI's buffer too small" );
    else
	memcpy( handle->bufadd, addr, handle->datatype->extent );
}


